@@ -34,18 +34,23 @@ class AppsCommand(Command):
             for app in apps:
                 print app.name
         else:
-            print "You don't have any apps yet. Run \"orchard apps create\" to create one."
+            log.error("You don't have any apps yet. Run \"orchard apps create\" to create one.")
 
     def create(self, options):
         """
         Create a new app.
         Usage: create NAME
         """
-        # TODO: handle invalid or clashing app name
         try:
             app = self.api.apps.create({"name": options['NAME']})
         except BadRequest as e:
-            print e.json
+            name_errors = e.json.get('name', None)
+
+            if name_errors:
+                log.error("\n".join(name_errors))
+            else:
+                log.error(e.json)
+
             sys.exit(1)
         log.info("Created %s", app.name)
 
