@@ -7,7 +7,6 @@ import fcntl
 import os
 import termios
 import threading
-import time
 import errno
 
 import logging
@@ -15,13 +14,18 @@ log = logging.getLogger(__name__)
 
 
 class SocketClient:
-    def __init__(self, socket, interactive, keep_running, raw=True):
-        self.socket = socket
-        self.interactive = interactive
-        self.keep_running = keep_running
+    def __init__(self,
+        socket_in=None,
+        socket_out=None,
+        socket_err=None,
+        raw=True,
+    ):
+        self.socket_in = socket_in
+        self.socket_out = socket_out
+        self.socket_err = socket_err
         self.raw = raw
+
         self.stdin_fileno = sys.stdin.fileno()
-        self.recv_thread = None
 
     def __enter__(self):
         self.create()
@@ -36,9 +40,10 @@ class SocketClient:
         else:
             self.settings = None
 
-        if self.interactive:
+        if self.socket_in is not None:
             self.set_blocking(sys.stdin, False)
             self.set_blocking(sys.stdout, True)
+            self.set_blocking(sys.stderr, True)
 
         if self.raw:
             tty.setraw(sys.stdin.fileno())
@@ -50,59 +55,58 @@ class SocketClient:
         fcntl.fcntl(fd, fcntl.F_SETFL, flags)
 
     def run(self):
-        if self.interactive:
-            thread = threading.Thread(target=self.send_ws)
-            thread.daemon = True
-            thread.start()
+        if self.socket_in is not None:
+            self.start_background_thread(target=self.send_ws, args=(self.socket_in, sys.stdin))
 
-        self.recv_thread = threading.Thread(target=self.recv_ws)
-        self.recv_thread.daemon = True
-        self.recv_thread.start()
+        recv_threads = []
 
-        self.alive_check()
+        if self.socket_out is not None:
+            recv_threads.append(self.start_background_thread(target=self.recv_ws, args=(self.socket_out, sys.stdout)))
 
-    def recv_ws(self):
+        if self.socket_err is not None:
+            recv_threads.append(self.start_background_thread(target=self.recv_ws, args=(self.socket_err, sys.stderr)))
+
+        for t in recv_threads:
+            t.join()
+
+    def start_background_thread(self, **kwargs):
+        thread = threading.Thread(**kwargs)
+        thread.daemon = True
+        thread.start()
+        return thread
+
+    def recv_ws(self, socket, stream):
         try:
             while True:
-                chunk = self.socket.recv()
+                chunk = socket.recv()
 
                 if chunk:
-                    sys.stdout.write(chunk)
-                    sys.stdout.flush()
+                    stream.write(chunk)
+                    stream.flush()
                 else:
                     break
         except Exception, e:
             log.debug(e)
 
-    def send_ws(self):
+    def send_ws(self, socket, stream):
         while True:
-            r, w, e = select([sys.stdin.fileno()], [], [])
+            r, w, e = select([stream.fileno()], [], [])
 
             if r:
-                chunk = sys.stdin.read(1)
+                chunk = stream.read(1)
 
                 if chunk == '':
-                    self.socket.send_close()
+                    socket.send_close()
                     break
                 else:
                     try:
-                        self.socket.send(chunk)
+                        socket.send(chunk)
                     except Exception, e:
                         if hasattr(e, 'errno') and e.errno == errno.EPIPE:
                             break
                         else:
                             raise e
 
-    def alive_check(self):
-        while True:
-            time.sleep(1)
-
-            if not self.recv_thread.is_alive():
-                break
-
-            if not self.keep_running():
-                break
-
     def destroy(self):
         if self.settings is not None:
             termios.tcsetattr(self.stdin_fileno, termios.TCSADRAIN, self.settings)
